@@ -189,6 +189,7 @@ class object(univention.admin.handlers.simpleLdap):
     }
 
     def open(self):
+        # type: () -> None
         univention.admin.handlers.simpleLdap.open(self)
 
         pathResult, self.default_dn = default_container_for_objects(self.lo, self.position.getDomain())
@@ -204,6 +205,7 @@ class object(univention.admin.handlers.simpleLdap):
         self.save()
 
     def _ldap_post_create(self):
+        # type: () -> None
         super(object, self)._ldap_post_create()
         changes = []
 
@@ -222,12 +224,14 @@ class object(univention.admin.handlers.simpleLdap):
             self.lo.modify(self.default_dn, changes)
 
     def _ldap_pre_rename(self, newdn):
+        # type: (str) -> None
         super(object, self)._ldap_pre_rename(newdn)
         self.move(newdn)
 
     def _ldap_post_move(self, olddn):
+        # type: (str) -> None
         super(object, self)._ldap_post_move(olddn)
-        settings_module = univention.admin.modules.get('settings/directory')
+        settings_module = univention.admin.modules._get('settings/directory')
         settings_object = univention.admin.objects.get(settings_module, None, self.lo, position='', dn=self.default_dn)
         settings_object.open()
         needs_modify = False
@@ -240,6 +244,7 @@ class object(univention.admin.handlers.simpleLdap):
             settings_object.modify()
 
     def _ldap_post_modify(self):
+        # type: () -> None
         super(object, self)._ldap_post_modify()
         changes = []
 
@@ -254,6 +259,7 @@ class object(univention.admin.handlers.simpleLdap):
             self.lo.modify(self.default_dn, changes)
 
     def _ldap_pre_remove(self):
+        # type: () -> None
         super(object, self)._ldap_pre_remove()
         changes = []
 
@@ -267,6 +273,7 @@ class object(univention.admin.handlers.simpleLdap):
 
     @classmethod
     def unmapped_lookup_filter(cls):
+        # type: () -> univention.admin.filter.conjunction
         return univention.admin.filter.conjunction('&', [
             univention.admin.filter.expression('objectClass', 'organizationalRole'),
             univention.admin.filter.conjunction('!', [univention.admin.filter.expression('cn', 'univention')]),
@@ -279,4 +286,5 @@ lookup_filter = object.lookup_filter
 
 
 def identify(dn, attr, canonical=False):
+    # type: (str, univention.admin.handlers._Attributes, bool) -> bool
     return b'organizationalRole' in attr.get('objectClass', []) and attr.get("cn", []) != [b"univention"] and b'univentionBase' not in attr.get('objectClass', [])
