@@ -33,12 +33,15 @@
 """|UDM| module for |DNS| forward zones"""
 
 import ipaddress
+from typing import Any  # noqa: F401
 
 import univention.admin.filter
 import univention.admin.handlers
 import univention.admin.localization
 from univention.admin import configRegistry
-from univention.admin.handlers.dns import ARPA_IP4, ARPA_IP6, escapeSOAemail, stripDot, unescapeSOAemail
+from univention.admin.handlers.dns import (  # noqa: F401
+    ARPA_IP4, ARPA_IP6, escapeSOAemail, is_dns, is_forward_zone, is_zone, stripDot, unescapeSOAemail,
+)
 from univention.admin.layout import Group, Tab
 
 
@@ -84,7 +87,7 @@ property_descriptions = {
         long_description=_('The email address of the person responsible for this zone.'),
         syntax=univention.admin.syntax.emailAddressThatMayEndWithADot,
         required=True,
-        default=('root@%s' % configRegistry.get('domainname'), []),
+        default=('root@%(domainname)s.' % configRegistry, []),
     ),
     'serial': univention.admin.property(
         short_description=_('Serial number'),
@@ -123,7 +126,7 @@ property_descriptions = {
     ),
     'nameserver': univention.admin.property(
         short_description=_('Name server'),
-        long_description=_('The FQDNs of the servers serving this zone.'),
+        long_description=_('The FQDNs of the servers serving this zone. A FQDN must end with a dot.'),
         syntax=univention.admin.syntax.dnsHostname,
         multivalue=True,
         required=True,
@@ -152,8 +155,8 @@ layout = [
     Tab(_('General'), _('Basic settings'), layout=[
         Group(_('General forward lookup zone settings'), layout=[
             'zone',
-            'nameserver',
             'zonettl',
+            'nameserver',
         ]),
     ]),
     Tab(_('Start of authority'), _('Primary name server information'), layout=[
@@ -183,17 +186,19 @@ layout = [
 
 
 def mapMX(old, encoding=()):
+    # type: (list[list[str]], univention.admin.handlers._Encoding) -> list[bytes]
     return [u' '.join(entry).encode(*encoding) for entry in old]
 
 
 def unmapMX(old, encoding=()):
+    # type: (list[bytes], univention.admin.handlers._Encoding) -> list[list[str]]
     return [entry.decode(*encoding).split(u' ', 1) for entry in old]
 
 
 mapping = univention.admin.mapping.mapping()
 mapping.register('zone', 'zoneName', stripDot, univention.admin.mapping.ListToString, encoding='ASCII')
-mapping.register('nameserver', 'nSRecord', encoding='ASCII')
 mapping.register('zonettl', 'dNSTTL', univention.admin.mapping.mapUNIX_TimeInterval, univention.admin.mapping.unmapUNIX_TimeInterval)
+mapping.register('nameserver', 'nSRecord', encoding='ASCII')
 mapping.register('mx', 'mXRecord', mapMX, unmapMX, encoding='ASCII')
 mapping.register('txt', 'tXTRecord', encoding='ASCII')
 
@@ -201,22 +206,33 @@ mapping.register('txt', 'tXTRecord', encoding='ASCII')
 class object(univention.admin.handlers.simpleLdap):
     module = module
 
-    def __init__(self, co, lo, position, dn='', superordinate=None, attributes=[]):
+    def __init__(
+        self,
+        co,  # type: None
+        lo,  # type: univention.admin.uldap.access
+        position,  # type: univention.admin.uldap.position | None
+        dn=u'',  # type: str
+        superordinate=None,  # type: univention.admin.handlers.simpleLdap | None
+        attributes=None,  # type: univention.admin.handlers._Attributes | None
+    ):  # type: (...) -> None
         univention.admin.handlers.simpleLdap.__init__(self, co, lo, position, dn, superordinate, attributes=attributes)
         if not self.dn and not self.position:
             raise univention.admin.uexceptions.insufficientInformation(_('Neither DN nor position given.'))
 
     def _post_unmap(self, info, values):
+        # type: (univention.admin.handlers._Properties, univention.admin.handlers._Attributes) -> univention.admin.handlers._Properties
         info = super(object, self)._post_unmap(info, values)
         info['a'] = []
         if 'aRecord' in values:
-            info['a'].extend([x.decode('ASCII') for x in values['aRecord']])
+            info['a'] += (x.decode('ASCII') for x in values['aRecord'])
         if 'aAAARecord' in values:
-            info['a'].extend([ipaddress.IPv6Address(x.decode('ASCII')).exploded for x in values['aAAARecord']])
+            info['a'] += (ipaddress.IPv6Address(x.decode('ASCII')).exploded for x in values['aAAARecord'])
         return info
 
     def open(self):
+        # type: () -> None
         univention.admin.handlers.simpleLdap.open(self)
+
         soa = self.oldattr.get('sOARecord', [b''])[0].split(b' ')
         if len(soa) > 6:
             self['contact'] = unescapeSOAemail(soa[1].decode('ASCII'))
@@ -229,20 +245,19 @@ class object(univention.admin.handlers.simpleLdap):
         self.save()
 
     def _ldap_addlist(self):
+        # type: () -> list[tuple[str, Any]]
         return super(object, self)._ldap_addlist() + [
             ('relativeDomainName', [b'@']),
         ]
 
     def _ldap_modlist(self):
+        # type: () -> list[tuple[str, Any, Any]]
         ml = univention.admin.handlers.simpleLdap._ldap_modlist(self)
         if self.hasChanged(['nameserver', 'contact', 'serial', 'refresh', 'retry', 'expire', 'ttl']):
             if self['contact'] and not self['contact'].endswith('.'):
                 self['contact'] += '.'
-            for i in range(len(self['nameserver'])):
-                if len(self['nameserver'][i]) > 0 \
-                        and ':' not in self['nameserver'][i] \
-                        and '.' in self['nameserver'][i] \
-                        and not self['nameserver'][i].endswith('.'):
+            for i, ns in enumerate(self['nameserver']):
+                if ns and ':' not in ns and '.' in ns and not ns.endswith('.'):
                     self['nameserver'][i] += '.'
             refresh = univention.admin.mapping.mapUNIX_TimeInterval(self['refresh'])
             retry = univention.admin.mapping.mapUNIX_TimeInterval(self['retry'])
@@ -275,6 +290,7 @@ class object(univention.admin.handlers.simpleLdap):
         return ml
 
     def _ldap_pre_modify(self):
+        # type: () -> None
         super(object, self)._ldap_pre_modify()
         # update SOA record
         if not self.hasChanged('serial'):
@@ -282,9 +298,10 @@ class object(univention.admin.handlers.simpleLdap):
 
     @classmethod
     def unmapped_lookup_filter(cls):
+        # type: () -> univention.admin.filter.conjunction
         return univention.admin.filter.conjunction('&', [
             univention.admin.filter.expression('objectClass', 'dNSZone'),
-            univention.admin.filter.expression('relativeDomainName', '@'),
+            univention.admin.filter.expression('sOARecord', '*', escape=False),
             univention.admin.filter.conjunction('!', [univention.admin.filter.expression('zoneName', '*%s' % ARPA_IP4, escape=False)]),
             univention.admin.filter.conjunction('!', [univention.admin.filter.expression('zoneName', '*%s' % ARPA_IP6, escape=False)]),
         ])
@@ -295,4 +312,9 @@ lookup_filter = object.lookup_filter
 
 
 def identify(dn, attr, canonical=False):
-    return b'dNSZone' in attr.get('objectClass', []) and [b'@'] == attr.get('relativeDomainName', []) and not attr['zoneName'][0].decode('ASCII').endswith(ARPA_IP4) and not attr['zoneName'][0].decode('ASCII').endswith(ARPA_IP6)
+    # type: (str, univention.admin.handlers._Attributes, bool) -> bool
+    return bool(
+        is_zone(attr)
+        and is_dns(attr)
+        and is_forward_zone(attr),
+    )
