@@ -32,13 +32,17 @@
 
 """|UDM| module for |DNS| reverse zones"""
 
+from typing import Any  # noqa: F401
+
 import ldap.dn
 
 import univention.admin.filter
 import univention.admin.handlers
 import univention.admin.localization
 from univention.admin import configRegistry
-from univention.admin.handlers.dns import ARPA_IP4, ARPA_IP6, escapeSOAemail, unescapeSOAemail
+from univention.admin.handlers.dns import (  # noqa: F401
+    ARPA_IP4, ARPA_IP6, escapeSOAemail, is_dns, is_reverse_zone, is_zone, unescapeSOAemail,
+)
 from univention.admin.layout import Group, Tab
 
 
@@ -49,7 +53,7 @@ module = 'dns/reverse_zone'
 operations = ['add', 'edit', 'remove', 'search']
 columns = ['nameserver']
 childs = True
-childmodules = ['dns/ptr_record']
+childmodules = ['dns/ptr_record', 'dns/ns_record']
 short_description = _('DNS: Reverse lookup zone')
 object_name = _('Reverse lookup zone')
 object_name_plural = _('Reverse lookup zones')
@@ -82,9 +86,9 @@ property_descriptions = {
     'contact': univention.admin.property(
         short_description=_('Contact person'),
         long_description=_('The email address of the person responsible for this zone.'),
-        syntax=univention.admin.syntax.string,
+        syntax=univention.admin.syntax.emailAddressThatMayEndWithADot,
         required=True,
-        default=('root@%s.' % configRegistry.get('domainname', ''), []),
+        default=('root@%(domainname)s.' % configRegistry, []),
     ),
     'serial': univention.admin.property(
         short_description=_('Serial number'),
@@ -123,7 +127,7 @@ property_descriptions = {
     ),
     'nameserver': univention.admin.property(
         short_description=_('Name server'),
-        long_description=_('The FQDNs of the servers serving this zone.'),
+        long_description=_('The FQDNs of the servers serving this zone. A FQDN must end with a dot.'),
         syntax=univention.admin.syntax.dnsHostname,
         multivalue=True,
         required=True,
@@ -201,18 +205,27 @@ mapping.register('nameserver', 'nSRecord', encoding='ASCII')
 class object(univention.admin.handlers.simpleLdap):
     module = module
 
-    def __init__(self, co, lo, position, dn='', superordinate=None, attributes=[]):
+    def __init__(
+        self,
+        co,  # type: None
+        lo,  # type: univention.admin.uldap.access
+        position,  # type: univention.admin.uldap.position | None
+        dn=u'',  # type: str
+        superordinate=None,  # type: univention.admin.handlers.simpleLdap | None
+        attributes=None,  # type: univention.admin.handlers._Attributes | None
+    ):  # type: (...) -> None
         univention.admin.handlers.simpleLdap.__init__(self, co, lo, position, dn, superordinate, attributes=attributes)
         if not self.dn and not self.position:
             raise univention.admin.uexceptions.insufficientInformation(_('Neither DN nor position given.'))
 
     def open(self):
+        # type: () -> None
         univention.admin.handlers.simpleLdap.open(self)
 
         soa = self.oldattr.get('sOARecord', [b''])[0].split(b' ')
         if len(soa) > 6:
-            self['contact'] = unescapeSOAemail(soa[1].decode('UTF-8'))
-            self['serial'] = soa[2].decode('UTF-8')
+            self['contact'] = unescapeSOAemail(soa[1].decode('ASCII'))
+            self['serial'] = soa[2].decode('ASCII')
             self['refresh'] = univention.admin.mapping.unmapUNIX_TimeInterval(soa[3])
             self['retry'] = univention.admin.mapping.unmapUNIX_TimeInterval(soa[4])
             self['expire'] = univention.admin.mapping.unmapUNIX_TimeInterval(soa[5])
@@ -221,31 +234,31 @@ class object(univention.admin.handlers.simpleLdap):
         self.save()
 
     def _ldap_modlist(self):
+        # type: () -> list[tuple[str, Any, Any]]
         ml = univention.admin.handlers.simpleLdap._ldap_modlist(self)
         if self.hasChanged(['nameserver', 'contact', 'serial', 'refresh', 'retry', 'expire', 'ttl']):
             if self['contact'] and not self['contact'].endswith('.'):
                 self['contact'] += '.'
-            for i in range(len(self['nameserver'])):
-                if len(self['nameserver'][i]) > 0 \
-                        and ':' not in self['nameserver'][i] \
-                        and '.' in self['nameserver'][i] \
-                        and not self['nameserver'][i].endswith('.'):
+            for i, ns in enumerate(self['nameserver']):
+                if ns and ':' not in ns and '.' in ns and not ns.endswith('.'):
                     self['nameserver'][i] += '.'
             refresh = univention.admin.mapping.mapUNIX_TimeInterval(self['refresh'])
             retry = univention.admin.mapping.mapUNIX_TimeInterval(self['retry'])
             expire = univention.admin.mapping.mapUNIX_TimeInterval(self['expire'])
             ttl = univention.admin.mapping.mapUNIX_TimeInterval(self['ttl'])
-            soa = b'%s %s %s %s %s %s %s' % (self['nameserver'][0].encode('UTF-8'), escapeSOAemail(self['contact']).encode('UTF-8'), self['serial'].encode('UTF-8'), refresh, retry, expire, ttl)
+            soa = b'%s %s %s %s %s %s %s' % (self['nameserver'][0].encode('ASCII'), escapeSOAemail(self['contact']).encode('ASCII'), self['serial'].encode('ASCII'), refresh, retry, expire, ttl)
             ml.append(('sOARecord', self.oldattr.get('sOARecord', []), soa))
         return ml
 
     def _ldap_pre_modify(self):
+        # type: () -> None
         super(object, self)._ldap_pre_modify()
         # update SOA record
         if not self.hasChanged('serial'):
             self['serial'] = str(int(self['serial']) + 1)
 
     def _ldap_addlist(self):
+        # type: () -> list[tuple[str, Any]]
         return super(object, self)._ldap_addlist() + [
             ('relativeDomainName', [b'@']),
         ]
@@ -253,17 +266,18 @@ class object(univention.admin.handlers.simpleLdap):
     # FIXME: there should be general solution; subnet is just a naming
     # attribute (though calculated from rdn)
     def description(self):
-        if 0:  # open?
-            return self['subnet']
-        else:
+        # type: () -> str
+        if self.exists():
             rdn_value = ldap.dn.str2dn(self.dn)[0][0][1].encode('UTF-8')
             return unmapSubnet([rdn_value])
+        return self['subnet']
 
     @classmethod
     def unmapped_lookup_filter(cls):
+        # type: () -> univention.admin.filter.conjunction
         return univention.admin.filter.conjunction('&', [
             univention.admin.filter.expression('objectClass', 'dNSZone'),
-            univention.admin.filter.expression('relativeDomainName', '@'),
+            univention.admin.filter.expression('sOARecord', '*', escape=False),
             univention.admin.filter.conjunction('|', [
                 univention.admin.filter.expression('zoneName', '*%s' % ARPA_IP4, escape=False),
                 univention.admin.filter.expression('zoneName', '*%s' % ARPA_IP6, escape=False),
@@ -276,6 +290,9 @@ lookup_filter = object.lookup_filter
 
 
 def identify(dn, attr):
-    return b'dNSZone' in attr.get('objectClass', []) and \
-        [b'@'] == attr.get('relativeDomainName', []) and \
-        (attr['zoneName'][0].decode('ASCII').endswith(ARPA_IP4) or attr['zoneName'][0].decode('ASCII').endswith(ARPA_IP6))
+    # type: (str, univention.admin.handlers._Attributes) -> bool
+    return bool(
+        is_zone(attr)
+        and is_dns(attr)
+        and is_reverse_zone(attr),
+    )
