@@ -32,8 +32,6 @@
 
 """|UDM| module for mailinglists"""
 
-import univention.admin.allocators
-import univention.admin.filter
 import univention.admin.handlers
 import univention.admin.localization
 from univention.admin.layout import Group, Tab
@@ -130,6 +128,7 @@ class object(univention.admin.handlers.simpleLdap):
     module = module
 
     def _ldap_pre_ready(self):
+        # type: () -> None
         super(object, self)._ldap_pre_ready()
         if not self.exists() or self.hasChanged('mailAddress'):
             try:
@@ -139,6 +138,7 @@ class object(univention.admin.handlers.simpleLdap):
                 raise univention.admin.uexceptions.mailAddressUsed(self['mailAddress'])
 
     def _ldap_pre_remove(self):
+        # type: () -> None
         super(object, self)._ldap_pre_remove()
         if self.oldattr.get('mailPrimaryAddress'):
             self.alloc.append(('mailPrimaryAddress', self.oldattr['mailPrimaryAddress'][0].decode('UTF-8')))
