@@ -35,7 +35,7 @@
 from __future__ import absolute_import
 
 from logging import getLogger
-from typing import Any, Dict, List, Optional, Tuple, Union  # noqa: F401
+from typing import Any  # noqa: F401
 
 import ldap
 
@@ -46,7 +46,7 @@ log = getLogger('ADMIN')
 
 
 def module(object):
-    # type: (univention.admin.handlers.simpleLdap) -> Optional[str]
+    # type: (univention.admin.handlers.simpleLdap) -> str | None
     """
     Return handler name for |UDM| object.
 
@@ -57,7 +57,7 @@ def module(object):
 
 
 def get_superordinate(module, co, lo, dn):
-    # type: (univention.admin.modules.UdmModule, None, univention.admin.uldap.access, str) -> Optional[univention.admin.handlers.simpleLdap]
+    # type: (univention.admin.modules.UdmModule, None, univention.admin.uldap.access, str) -> univention.admin.handlers.simpleLdap | None
     """
     Searches for the superordinate object for the given DN.
 
@@ -71,9 +71,9 @@ def get_superordinate(module, co, lo, dn):
     if super_modules:
         while dn:
             attr = lo.get(dn)
-            super_module = {univention.admin.modules.name(x) for x in univention.admin.modules.identify(dn, attr)} & super_modules
-            if super_module:
-                super_module = univention.admin.modules.get(list(super_module)[0])  # noqa: RUF015
+            modules = {univention.admin.modules.name(x) for x in univention.admin.modules.identify(dn, attr)} & super_modules
+            if modules:
+                super_module = univention.admin.modules._get(list(modules)[0])  # noqa: RUF015
                 return get(super_module, co, lo, None, dn)
             dn = lo.parentDn(dn)
 
@@ -81,7 +81,7 @@ def get_superordinate(module, co, lo, dn):
 
 
 def get(module, co, lo, position, dn='', attr=None, superordinate=None, attributes=None):
-    # type: (univention.admin.modules.UdmModule, None, univention.admin.uldap.access, univention.admin.uldap.position, str, Dict[str, List[Any]], Any, Any) -> univention.admin.handlers.simpleLdap
+    # type: (univention.admin.modules.UdmModule, None, univention.admin.uldap.access, univention.admin.uldap.position, str, dict[str, list[Any]] | None, Any | None, Any | None) -> univention.admin.handlers.simpleLdap | None
     """
     Return object of module while trying to create objects of
     superordinate modules as well.
@@ -137,7 +137,7 @@ def default(module, co, lo, position):
     :param position: |UDM| position instance.
     :returns: An initialized |UDM| object.
     """
-    module = univention.admin.modules.get(module)
+    module = univention.admin.modules._get(module)
     object = module.object(co, lo, position)
     for name, property in module.property_descriptions.items():
         default = property.default(object)
@@ -157,7 +157,7 @@ def description(object):
 
 
 def shadow(lo, module, object, position):
-    # type: (univention.admin.uldap.access, univention.admin.modules.UdmModule, univention.admin.handlers.simpleLdap, univention.admin.uldap.position) -> Union[Tuple[univention.admin.handlers.simpleLdap, univention.admin.modules.UdmModule], Tuple[None, None]]
+    # type: (univention.admin.uldap.access, univention.admin.modules.UdmModule, univention.admin.handlers.simpleLdap, univention.admin.uldap.position) -> tuple[univention.admin.handlers.simpleLdap, univention.admin.modules.UdmModule] | tuple[None, None]
     """
     If object is a container, return object and module the container
     shadows (that is usually the one that is subordinate in the LDAP tree).
@@ -186,7 +186,7 @@ def shadow(lo, module, object, position):
 
 
 def dn(object):
-    # type: (univention.admin.handlers.simpleLdap) -> Optional[str]
+    # type: (univention.admin.handlers.simpleLdap) -> str | None
     """
     Return the |DN| of the object.
 
@@ -197,7 +197,7 @@ def dn(object):
 
 
 def ocToType(oc):
-    # type: (str) -> Optional[str]
+    # type: (str) -> str | None
     """
     Return the |UDM| module capabale of handling the given |LDAP| objectClass.
 
@@ -241,7 +241,7 @@ def emptyAttribute(object, key):
 
 
 def getPolicyReference(object, policy_type):
-    # type: (univention.admin.handlers.simpleLdap, str) -> Optional[univention.admin.handlers.simplePolicy]
+    # type: (univention.admin.handlers.simpleLdap, str) -> univention.admin.handlers.simplePolicy | None
     """
     Return the policy of the requested type.
 
@@ -290,7 +290,7 @@ def replacePolicyReference(object, policy_type, new_reference):
     """
     # FIXME: Move this to handlers.simpleLdap?
 
-    module = univention.admin.modules.get(policy_type)
+    module = univention.admin.modules._get(policy_type)
     if not univention.admin.modules.recognize(module, new_reference, object.lo.get(new_reference)):
         log.debug('replacePolicyReference: error.')
         return
@@ -310,8 +310,9 @@ def restorePolicyReference(object, policy_type):
     :param policy_type: Name of the |UDM| policy to lookup.
     """
     # FIXME: Move this to handlers.simpleLdap?
-    module = univention.admin.modules.get(policy_type)
-    if not module:
+    try:
+        module = univention.admin.modules._get(policy_type)
+    except LookupError:
         return
 
     removePolicyReference(object, policy_type)
@@ -337,7 +338,7 @@ def wantsCleanup(object):
     wantsCleanup = False
 
     object_module = module(object)
-    object_module = univention.admin.modules.get(object_module)
+    object_module = univention.admin.modules._get(object_module)
     if hasattr(object_module, 'docleanup'):
         wantsCleanup = object_module.docleanup
 
