@@ -116,7 +116,21 @@ from .udm import UDM
 
 __all__ = [
     'UDM',
-    'CreateError', 'DeleteError', 'DeletedError', 'NotYetSavedError', 'ModifyError', 'MoveError', 'MultipleObjects',
-    'NoObject', 'UdmError', 'UnknownProperty', 'UnknownModuleType', 'WrongObjectType', 'ConnectionError', 'NoSuperordinate',
-    'NoApiVersionSet', 'ApiVersionNotSupported', 'ApiVersionMustNotChange',
+    'ApiVersionMustNotChange',
+    'ApiVersionNotSupported',
+    'ConnectionError',
+    'CreateError',
+    'DeleteError',
+    'DeletedError',
+    'ModifyError',
+    'MoveError',
+    'MultipleObjects',
+    'NoApiVersionSet',
+    'NoObject',
+    'NoSuperordinate',
+    'NotYetSavedError',
+    'UdmError',
+    'UnknownModuleType',
+    'UnknownProperty',
+    'WrongObjectType',
 ]
