@@ -38,6 +38,7 @@ import datetime
 import logging
 import sys
 import time
+from typing import Any, Callable, Dict, List, Optional, Type, TypeVar  # noqa: F401
 
 import lazy_object_proxy
 import six
@@ -59,15 +60,19 @@ class BaseEncoder(object):
     static = False  # whether to create an instance or use a class/static method
 
     def __init__(self, property_name=None, *args, **kwargs):
+        # type: (str, *Any, **Any) -> None
         self.property_name = property_name
 
     def __repr__(self):
+        # type: () -> str
         return '{}({})'.format(self.__class__.__name__, self.property_name)
 
     def encode(self, value=None):
+        # type: (Optional[Any]) -> Optional[Any]
         raise NotImplementedError()
 
     def decode(self, value=None):
+        # type: (Optional[Any]) -> Optional[Any]
         raise NotImplementedError()
 
 
@@ -75,12 +80,14 @@ class Base64BinaryPropertyEncoder(BaseEncoder):
     static = False
 
     def decode(self, value=None):
+        # type: (Optional[str]) -> Optional[Base64BinaryProperty]
         if value:
             return Base64BinaryProperty(self.property_name, value)
         else:
             return value
 
     def encode(self, value=None):
+        # type: (Optional[Base64BinaryProperty]) -> Optional[str]
         if value:
             if not isinstance(value, Base64BinaryProperty):
                 value = Base64BinaryProperty(self.property_name, raw_value=value)
@@ -93,12 +100,14 @@ class Base64Bzip2BinaryPropertyEncoder(BaseEncoder):
     static = False
 
     def decode(self, value=None):
+        # type: (Optional[str]) -> Optional[Base64Bzip2BinaryProperty]
         if value:
             return Base64Bzip2BinaryProperty(self.property_name, value)
         else:
             return value
 
     def encode(self, value=None):
+        # type: (Optional[Base64Bzip2BinaryProperty]) -> Optional[str]
         if value:
             return value.encoded
         else:
@@ -110,6 +119,7 @@ class DatePropertyEncoder(BaseEncoder):
 
     @staticmethod
     def decode(value=None):
+        # type: (Optional[str]) -> Optional[datetime.date]
         if value:
             return datetime.date(*time.strptime(value, '%Y-%m-%d')[0:3])
         else:
@@ -117,6 +127,7 @@ class DatePropertyEncoder(BaseEncoder):
 
     @staticmethod
     def encode(value=None):
+        # type: (Optional[datetime.date]) -> Optional[str]
         if value:
             return value.strftime('%Y-%m-%d')
         else:
@@ -128,10 +139,12 @@ class DisabledPropertyEncoder(BaseEncoder):
 
     @staticmethod
     def decode(value=None):
+        # type: (Optional[str]) -> bool
         return value == '1'
 
     @staticmethod
     def encode(value=None):
+        # type: (Optional[bool]) -> str
         return '1' if value else '0'
 
 
@@ -140,6 +153,7 @@ class HomePostalAddressPropertyEncoder(BaseEncoder):
 
     @staticmethod
     def decode(value=None):
+        # type: (Optional[List[List[str]]]) -> Optional[List[Dict[str, str]]]
         if value:
             return [{'street': v[0], 'zipcode': v[1], 'city': v[2]} for v in value]
         else:
@@ -147,6 +161,7 @@ class HomePostalAddressPropertyEncoder(BaseEncoder):
 
     @staticmethod
     def encode(value=None):
+        # type: (Optional[List[Dict[str, str]]]) -> Optional[List[List[str]]]
         if value:
             return [[v['street'], v['zipcode'], v['city']] for v in value]
         else:
@@ -158,12 +173,14 @@ class ListOfListOflTextToDictPropertyEncoder(BaseEncoder):
 
     @staticmethod
     def decode(value=None):
+        # type: (Optional[List[List[str]]]) -> Optional[Dict[str, str]]
         if value is None:
             return value
         return dict(value)
 
     @staticmethod
     def encode(value=None):
+        # type: (Optional[Dict[str, str]]) -> Optional[List[List[str]]]
         if value:
             return [[k, v] for k, v in value.items()]
         else:
@@ -175,6 +192,7 @@ class MultiLanguageTextAppcenterPropertyEncoder(BaseEncoder):
 
     @staticmethod
     def decode(value=None):
+        # type: (Optional[List[str]]) -> Optional[Dict[str, str]]
         if value:
             res = {}
             for s in value:
@@ -187,6 +205,7 @@ class MultiLanguageTextAppcenterPropertyEncoder(BaseEncoder):
 
     @staticmethod
     def encode(value=None):
+        # type: (Optional[Dict[str, str]]) -> Optional[List[str]]
         if value:
             return ['[{}] {}'.format(k, v) for k, v in value.items()]
         else:
@@ -200,6 +219,7 @@ class SambaGroupTypePropertyEncoder(BaseEncoder):
 
     @classmethod
     def decode(cls, value=None):
+        # type: (Optional[List[str]]) -> Optional[str]
         try:
             return cls.choices[value]
         except KeyError:
@@ -207,6 +227,7 @@ class SambaGroupTypePropertyEncoder(BaseEncoder):
 
     @classmethod
     def encode(cls, value=None):
+        # type: (Optional[str]) -> Optional[List[str]]
         try:
             return cls.choices_reverted[value]
         except KeyError:
@@ -219,13 +240,15 @@ class SambaLogonHoursPropertyEncoder(BaseEncoder):
 
     @classmethod
     def decode(cls, value=None):
+        # type: (Optional[List[int]]) -> Optional[List[str]]
         if value:
-            return ['{} {}-{}'.format(cls._weekdays[v / 24], v % 24, v % 24 + 1) for v in value]
+            return ['{} {}-{}'.format(cls._weekdays[dow], hour, hour + 1) for dow, hour in (divmod(v, 24) for v in value)]
         else:
             return value
 
     @classmethod
     def encode(cls, value=None):
+        # type: (Optional[List[str]]) -> Optional[List[int]]
         if value:
             try:
                 values = [v.split() for v in value]
@@ -244,10 +267,12 @@ class StringCaseInsensitiveResultLowerBooleanPropertyEncoder(BaseEncoder):
 
     @classmethod
     def decode(cls, value=''):
+        # type: (Optional[str]) -> bool
         return isinstance(value, six.string_types) and value.lower() == cls.true_string
 
     @classmethod
     def encode(cls, value=None):
+        # type: (Optional[bool]) -> str
         assert cls.result_case_func in ('lower', 'upper')
         if value:
             return getattr(cls.true_string, cls.result_case_func)()
@@ -264,10 +289,12 @@ class StringIntBooleanPropertyEncoder(BaseEncoder):
 
     @staticmethod
     def decode(value=None):
+        # type: (Optional[str]) -> bool
         return value == '1'
 
     @staticmethod
     def encode(value=None):
+        # type: (Optional[bool]) -> str
         if value:
             return '1'
         else:
@@ -278,6 +305,7 @@ class StringIntPropertyEncoder(BaseEncoder):
     static = False
 
     def decode(self, value=None):
+        # type: (Optional[str]) -> Optional[int]
         if value in ('', None):
             return None
         else:
@@ -288,6 +316,7 @@ class StringIntPropertyEncoder(BaseEncoder):
 
     @staticmethod
     def encode(value=None):
+        # type: (Optional[int]) -> Optional[str]
         if value is None:
             return value
         else:
@@ -300,6 +329,7 @@ class StringListToList(BaseEncoder):
 
     @classmethod
     def decode(cls, value=None):
+        # type: (Optional[str]) -> Optional[List[str]]
         if value:
             return value.split(cls.separator)
         else:
@@ -307,6 +337,7 @@ class StringListToList(BaseEncoder):
 
     @classmethod
     def encode(cls, value=None):
+        # type: (Optional[List[str]]) -> Optional[str]
         if value:
             return cls.separator.join(value)
         else:
@@ -336,9 +367,11 @@ class DnListPropertyEncoder(BaseEncoder):
     class MyProxy(lazy_object_proxy.Proxy):
         # overwrite __repr__ for better navigation in ipython
         def __repr__(self, __getattr__=object.__getattribute__):
+            # type: (Callable[[object, str], object]) -> str
             return super(DnListPropertyEncoder.MyProxy, self).__str__()
 
     def __init__(self, property_name=None, connection=None, api_version=None, *args, **kwargs):
+        # type: (Optional[str], Optional[Any], Optional[int], *Any, **Any) -> None
         assert connection is not None, 'Argument "connection" must not be None.'
         assert api_version is not None, 'Argument "api_version" must not be None.'
         super(DnListPropertyEncoder, self).__init__(property_name, *args, **kwargs)
@@ -364,6 +397,7 @@ class DnListPropertyEncoder(BaseEncoder):
         return res
 
     def decode(self, value=None):
+        # type: (Optional[List[str]]) -> Optional[List[str]]
         if value is None:
             value = []
         assert hasattr(value, '__iter__'), 'Value is not iterable: {!r}'.format(value)
@@ -373,6 +407,7 @@ class DnListPropertyEncoder(BaseEncoder):
 
     @staticmethod
     def encode(value=None):
+        # type: (Optional[List[str]]) -> Optional[List[str]]
         try:
             del value.objs
         except AttributeError:
@@ -381,6 +416,7 @@ class DnListPropertyEncoder(BaseEncoder):
 
     @property
     def udm(self):
+        # type: () -> object
         return self._udm
 
 
@@ -388,6 +424,7 @@ class PoliciesEncoder(BaseEncoder):
     static = False
 
     def __init__(self, property_name=None, connection=None, api_version=None, module_name=None, *args, **kwargs):
+        # type: (Optional[str], Optional[Any], Optional[int], Optional[str], *Any, **Any) -> None
         assert connection is not None, 'Argument "connection" must not be None.'
         assert api_version is not None, 'Argument "api_version" must not be None.'
         super(PoliciesEncoder, self).__init__(property_name, *args, **kwargs)
@@ -395,6 +432,7 @@ class PoliciesEncoder(BaseEncoder):
         self.module_name = module_name
 
     def decode(self, value=None):
+        # type: (Optional[Any]) -> Dict[Any, List[Any]]
         policies = {}
         policy_modules = univention.admin.modules.policyTypes(self.module_name)
         if not policy_modules and self._udm.get(self.module_name)._orig_udm_module.childs:  # container, which allows every policy-type
@@ -412,6 +450,7 @@ class PoliciesEncoder(BaseEncoder):
         return policies
 
     def encode(self, value=None):
+        # type: (Optional[Dict[Any, List[Any]]]) -> List[Any]
         if value:
             return [y for x in value.values() for y in x]
         else:
@@ -534,15 +573,18 @@ class DnPropertyEncoder(BaseEncoder):
     class MyProxy(lazy_object_proxy.Proxy):
         # overwrite __repr__ for better navigation in ipython
         def __repr__(self, __getattr__=object.__getattribute__):
+            # type: (Callable[[object, str], object]) -> str
             return super(DnPropertyEncoder.MyProxy, self).__str__()
 
     def __init__(self, property_name=None, connection=None, api_version=None, *args, **kwargs):
+        # type: (str, Any, Optional[int], *Any, **Any) -> None
         assert connection is not None, 'Argument "connection" must not be None.'
         assert api_version is not None, 'Argument "api_version" must not be None.'
         super(DnPropertyEncoder, self).__init__(property_name, *args, **kwargs)
         self._udm = UDM(connection, api_version)
 
     def _dn_to_udm_object(self, value):
+        # type: (Any) -> Optional[Any]
         try:
             if self.udm_module_name == 'auto':
                 return self.udm.obj_by_dn(value)
@@ -556,6 +598,7 @@ class DnPropertyEncoder(BaseEncoder):
         return None
 
     def decode(self, value=None):
+        # type: (Optional[str]) -> Optional[str]
         if value in (None, ''):
             return None
         new_str = self.DnStr(value)
@@ -565,6 +608,7 @@ class DnPropertyEncoder(BaseEncoder):
 
     @staticmethod
     def encode(value=None):
+        # type: (Optional[str]) -> Optional[str]
         try:
             del value.obj
         except AttributeError:
@@ -573,15 +617,18 @@ class DnPropertyEncoder(BaseEncoder):
 
     @property
     def udm(self):
+        # type: () -> UDM
         return self._udm
 
 
 def _classify_name(name):
+    # type: (str) -> str
     mod_parts = name.split('/')
     return ''.join('{}{}'.format(mp[0].upper(), mp[1:]) for mp in mod_parts)
 
 
 def dn_list_property_encoder_for(udm_module_name):
+    # type: (str) -> Type[DnListPropertyEncoder]
     """
     Create a (cached) subclass of DnListPropertyEncoder specific for each UDM
     module.
@@ -590,17 +637,17 @@ def dn_list_property_encoder_for(udm_module_name):
             `auto` if auto-detection should be done. Auto-detection requires one
             additional LDAP-query per object (still lazy though)!
     :return: subclass of DnListPropertyEncoder
-    :rtype: type(DnListPropertyEncoder)
     """
     if udm_module_name not in __dn_list_property_encoder_class_cache:
         cls_name = str('DnListPropertyEncoder{}').format(_classify_name(udm_module_name))
         specific_encoder_cls = type(cls_name, (DnListPropertyEncoder,), {})
-        specific_encoder_cls.udm_module_name = udm_module_name
+        specific_encoder_cls.udm_module_name = udm_module_name  # type: ignore[attr-defined]
         __dn_list_property_encoder_class_cache[udm_module_name] = specific_encoder_cls
     return __dn_list_property_encoder_class_cache[udm_module_name]
 
 
 def dn_property_encoder_for(udm_module_name):
+    # type: (str) -> Type[DnPropertyEncoder]
     """
     Create a (cached) subclass of DnPropertyEncoder specific for each UDM
     module.
@@ -609,11 +656,10 @@ def dn_property_encoder_for(udm_module_name):
             `auto` if auto-detection should be done. Auto-detection requires one
             additional LDAP-query per object (still lazy though)!
     :return: subclass of DnPropertyEncoder
-    :rtype: type(DnPropertyEncoder)
     """
     if udm_module_name not in __dn_property_encoder_class_cache:
         cls_name = str('DnPropertyEncoder{}').format(_classify_name(udm_module_name))
         specific_encoder_cls = type(cls_name, (DnPropertyEncoder,), {})
-        specific_encoder_cls.udm_module_name = udm_module_name
+        specific_encoder_cls.udm_module_name = udm_module_name  # type: ignore[attr-defined]
         __dn_property_encoder_class_cache[udm_module_name] = specific_encoder_cls
     return __dn_property_encoder_class_cache[udm_module_name]
