@@ -47,7 +47,7 @@ import univention.admin.uldap
 univention.admin.modules.update()
 
 
-def change(username, password):
+def change(username, password):  # type: (str, str) -> None
     """
     Change the password of the given user
 
@@ -60,7 +60,7 @@ def change(username, password):
     except Exception:
         lo, pos = univention.admin.uldap.getMachineConnection()
 
-    module = univention.admin.modules.get('users/user')
+    module = univention.admin.modules._get('users/user')
 
     univention.admin.modules.init(lo, pos, module)
 
